/*
FILE : FIR2LIFT.C

Factor a wavelet FIR filter pair into all possible combinations of
lifting steps.

(C) C. Valens

Created     : 24/09/1999
Last update : 27/09/1999

Changes to this file
====================
02/12/2004, Leonard Bradfield (lhwb@golden.net)
            - new wavelet: LHWB_75
            - detailed usage explications in main.
02/12/2004, CV
            - Fixed argv bug.
*/


#include "redir.h"
#include "zpoly.h"
#include "factor.h"

#include <stdlib.h>
#include <stdio.h>



zpoly_pair LHWB_75(void)
{
  zpoly_pair fir = {NULL,NULL};

  fir.p1 = zpoly_add_element_to(fir.p1,0,3);
  fir.p1 = zpoly_add_element_to(fir.p1,-1,-18);
  fir.p1 = zpoly_add_element_to(fir.p1,-2,38);
  fir.p1 = zpoly_add_element_to(fir.p1,-3,-18);
  fir.p1 = zpoly_add_element_to(fir.p1,-4,3);
  zpoly_scale(fir.p1,1);

  fir.p2 = zpoly_add_element_to(fir.p2,0,-1);
  fir.p2 = zpoly_add_element_to(fir.p2,-1,6);
  fir.p2 = zpoly_add_element_to(fir.p2,-2,-15);
  fir.p2 = zpoly_add_element_to(fir.p2,-3,20);
  fir.p2 = zpoly_add_element_to(fir.p2,-4,-15);
  fir.p2 = zpoly_add_element_to(fir.p2,-5,6);
  fir.p2 = zpoly_add_element_to(fir.p2,-6,-1);
  zpoly_scale(fir.p2,1.0);

  return fir;
}

zpoly_pair CDF42(void)
{
  zpoly_pair fir = {NULL,NULL};

  fir.p1 = zpoly_add_element_to(fir.p1,-3,3);
  fir.p1 = zpoly_add_element_to(fir.p1,-2,-12);
  fir.p1 = zpoly_add_element_to(fir.p1,-1,5);
  fir.p1 = zpoly_add_element_to(fir.p1,0,40);
  fir.p1 = zpoly_add_element_to(fir.p1,1,5);
  fir.p1 = zpoly_add_element_to(fir.p1,2,-12);
  fir.p1 = zpoly_add_element_to(fir.p1,3,3);
  zpoly_scale(fir.p1,0.03125);

  fir.p2 = zpoly_add_element_to(fir.p2,-3,1);
  fir.p2 = zpoly_add_element_to(fir.p2,-2,-4);
  fir.p2 = zpoly_add_element_to(fir.p2,-1,6);
  fir.p2 = zpoly_add_element_to(fir.p2,0,-4);
  fir.p2 = zpoly_add_element_to(fir.p2,1,1);
  zpoly_scale(fir.p2,0.0625);

  return fir;
}


zpoly_pair DD22(void)
{
  zpoly_pair fir = {NULL,NULL};

  fir.p1 = zpoly_add_element_to(fir.p1,-2,-0.125);
  fir.p1 = zpoly_add_element_to(fir.p1,-1,0.25);
  fir.p1 = zpoly_add_element_to(fir.p1,0,0.75);
  fir.p1 = zpoly_add_element_to(fir.p1,1,0.25);
  fir.p1 = zpoly_add_element_to(fir.p1,2,-0.125);

  fir.p2 = zpoly_add_element_to(fir.p2,-2,0.25);
  fir.p2 = zpoly_add_element_to(fir.p2,-1,-0.5);
  fir.p2 = zpoly_add_element_to(fir.p2,0,0.25);

  return fir;
}


int main(int argc, char *argv[])
{
  zpoly_pair fir;
  factor_list *fl;

	// Assign your wavelet filters here. This line will
	// make the program factor the LHWB_75 wavelet filters.
  fir = LHWB_75();

  printf("   FIR2LIFT, (C) C. Valens, 1999-2004.\n");
  printf("   Factor a wavelet FIR filter pair into lifting steps.\n\n");
  
  printf("\n*** How to use the results:\n");
  printf("\nOpen the output file in a text editor.\n");
  printf("\nYour analyzing polyphase matrix \n");
  printf("         [h(z)]\n");
  printf("         [g(z)]\n");
  printf("(h(z) lowpass, g(z) highpass)\n");
  printf("is broken into even-odd form\n");
  printf("         [heven(z)     hodd(z)]\n");
  printf("         [geven(z)     godd(z)],\n");
  printf("following *** 1: (and indicated below as M(1:))\n");
  printf("It is the product of usually several decompositions,\n");
  printf("each listed as a group (e.g. *** 2:).\n");
  printf("A specific decomposition is the matrix product\n");
  printf("\n\tM(1:) = M(1)*M(2)...M(n)\n");
  printf("\nwhere M(1) is the first listed matrix in the group\n");
  printf("and M(n) is the last.\n");
  printf("Standard [row,col] indexing is used.\n\n");
  printf("To change the coefficient display precision, change the format\n");
  printf("in redir_printf(..,fabs(p->coeff)) near line 62 in zpoly.c\n");
  printf("High precision is useful for converting to fractions\n");
  printf("using for example Maxima's (or MACSYMA's) RAT() routine.\n");

	// Usage: fir2lift [output_stream [stream_parameters]]
	// Examples:
	//
	//   fir2lift                  Output is sent to CON(sole), grab it if you can.
	//
	//   fir2lift factors.txt      Output is sent to file "factors.txt".
	//                             If "factors.txt" does not exist it is created.
	//                             If "factors.txt" exists it is overwritten.
	//
	//   fir2lift factors.txt a+   Output is appended to file "factors.txt".
	//                             If "factors.txt" does not exist it is created.

  if ((argc<2) || (argc>3)) {
    redir_open(0,0);
  }
  else if (argc==2) {
    redir_open(argv[1],0);
  }
  else if (argc==3) {
    redir_open(argv[1],argv[2]);
  }

  redir_printf("*** FIR2LIFT, (C) C. Valens, 1999-2004.\n\n");

  redir_printf("*** The filters to factor:\n");
  redir_printf("*** h(z) = [");
  zpoly_write(fir.p1);
  redir_printf("]\n");
  redir_printf("*** g(z) = [");
  zpoly_write(fir.p2);
  redir_printf("]\n\n");

  fl = factor_this_baby_now(fir);

  redir_printf("\n/*********************************************/");
  redir_printf("\n/*********************************************/");
  redir_printf("\n*** How to use the results:\n");
  redir_printf("\nYour analyzing polyphase matrix \n");
  redir_printf("         [h(z)]\n");
  redir_printf("         [g(z)]\n");
  redir_printf("(h(z) lowpass, g(z) highpass)\n");
  redir_printf("is broken into even-odd form\n");
  redir_printf("         [heven(z)     hodd(z)]\n");
  redir_printf("         [geven(z)     godd(z)],\n");
  redir_printf("following *** 1: (and indicated below as M(1:))\n");
  redir_printf("It is the product of usually several decompositions,\n");
  redir_printf("each listed as a group (e.g. *** 2:).\n");
  redir_printf("A specific decomposition is the matrix product\n");
  redir_printf("\n\tM(1:) = M(1)*M(2)...M(n)\n");
  redir_printf("\nwhere M(1) is the first listed matrix in the group\n");
  redir_printf("and M(n) is the last.\n");
  redir_printf("Standard [row,col] indexing is used.\n\n");
  redir_printf("To change the coefficient display precision, change the format\n");
  redir_printf("in redir_printf(..,fabs(p->coeff)) near line 62 in zpoly.c\n");
  redir_printf("High precision is useful for converting to fractions\n");
  redir_printf("using for example Maxima's (or MACSYMA's) RAT() routine.\n");
  redir_printf("\n/*********************************************/");
  redir_printf("\n/*********************************************/\n\n\n");

  gimme_my_lifting_steps(fl);

  redir_printf("\n*** Done.\n");
  redir_close();

  return 0;
}
